/* Generated by wayland-scanner 1.20.0 */

#ifndef KWINFT_OUTPUT_DEVICE_UNSTABLE_V1_SERVER_PROTOCOL_H
#define KWINFT_OUTPUT_DEVICE_UNSTABLE_V1_SERVER_PROTOCOL_H

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

#ifdef  __cplusplus
extern "C" {
#endif

struct wl_client;
struct wl_resource;

/**
 * @page page_kwinft_output_device_unstable_v1 The kwinft_output_device_unstable_v1 protocol
 * protocol to obtain information about output devices
 *
 * @section page_desc_kwinft_output_device_unstable_v1 Description
 *
 * This protocol exposes an interface for globally announced output devices.
 *
 * Warning! The protocol described in this file is experimental and
 * backward incompatible changes may be made.
 *
 * @section page_ifaces_kwinft_output_device_unstable_v1 Interfaces
 * - @subpage page_iface_zkwinft_output_device_v1 - output configuration representation
 * @section page_copyright_kwinft_output_device_unstable_v1 Copyright
 * <pre>
 *
 * Copyright © 2015 Sebastian Kügler <sebas@kde.org>
 * Copyright © 2020 Roman Gilg <subdiff@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 * </pre>
 */
struct zkwinft_output_device_v1;

#ifndef ZKWINFT_OUTPUT_DEVICE_V1_INTERFACE
#define ZKWINFT_OUTPUT_DEVICE_V1_INTERFACE
/**
 * @page page_iface_zkwinft_output_device_v1 zkwinft_output_device_v1
 * @section page_iface_zkwinft_output_device_v1_desc Description
 *
 * An output device describes a display device available to the compositor.
 * output_device is similar to wl_output, but focuses on output
 * configuration management.
 *
 * A client can query all global output_device objects to enlist all
 * available display devices, even those that may currently not be
 * represented by the compositor as a wl_output.
 *
 * The client sends configuration changes to the server through the
 * output_configuration interface, and the server applies the configuration
 * changes to the hardware and signals changes to the output devices
 * accordingly.
 *
 * This object is published as global during start up for every available
 * display devices, or when one later becomes available, for example by
 * being hotplugged via a physical connector.
 * @section page_iface_zkwinft_output_device_v1_api API
 * See @ref iface_zkwinft_output_device_v1.
 */
/**
 * @defgroup iface_zkwinft_output_device_v1 The zkwinft_output_device_v1 interface
 *
 * An output device describes a display device available to the compositor.
 * output_device is similar to wl_output, but focuses on output
 * configuration management.
 *
 * A client can query all global output_device objects to enlist all
 * available display devices, even those that may currently not be
 * represented by the compositor as a wl_output.
 *
 * The client sends configuration changes to the server through the
 * output_configuration interface, and the server applies the configuration
 * changes to the hardware and signals changes to the output devices
 * accordingly.
 *
 * This object is published as global during start up for every available
 * display devices, or when one later becomes available, for example by
 * being hotplugged via a physical connector.
 */
extern const struct wl_interface zkwinft_output_device_v1_interface;
#endif

#ifndef ZKWINFT_OUTPUT_DEVICE_V1_ENABLEMENT_ENUM
#define ZKWINFT_OUTPUT_DEVICE_V1_ENABLEMENT_ENUM
/**
 * @ingroup iface_zkwinft_output_device_v1
 * describes enabled state
 *
 * Describes whether a device is enabled, i.e. device is used to
 * display content by the compositor. This wraps a boolean around
 * an int to avoid a boolean trap.
 */
enum zkwinft_output_device_v1_enablement {
	ZKWINFT_OUTPUT_DEVICE_V1_ENABLEMENT_DISABLED = 0,
	ZKWINFT_OUTPUT_DEVICE_V1_ENABLEMENT_ENABLED = 1,
};
#endif /* ZKWINFT_OUTPUT_DEVICE_V1_ENABLEMENT_ENUM */

#ifndef ZKWINFT_OUTPUT_DEVICE_V1_MODE_FLAG_ENUM
#define ZKWINFT_OUTPUT_DEVICE_V1_MODE_FLAG_ENUM
/**
 * @ingroup iface_zkwinft_output_device_v1
 * mode information
 *
 * These flags describe properties of an output mode. They are
 * used in the flags bitfield of the mode event.
 */
enum zkwinft_output_device_v1_mode_flag {
	/**
	 * indicates this is the current mode
	 */
	ZKWINFT_OUTPUT_DEVICE_V1_MODE_FLAG_CURRENT = 0,
	/**
	 * indicates this is the preferred mode
	 */
	ZKWINFT_OUTPUT_DEVICE_V1_MODE_FLAG_PREFERRED = 1,
};
#endif /* ZKWINFT_OUTPUT_DEVICE_V1_MODE_FLAG_ENUM */

#ifndef ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_ENUM
#define ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_ENUM
/**
 * @ingroup iface_zkwinft_output_device_v1
 * transform from framebuffer to output
 *
 * This describes the transform, that a compositor will apply to a
 * surface to compensate for the rotation or mirroring of an
 * output device.
 *
 * The flipped values correspond to an initial flip around a
 * vertical axis followed by rotation.
 *
 * The purpose is mainly to allow clients to render accordingly and
 * tell the compositor, so that for fullscreen surfaces, the
 * compositor is still able to scan out directly client surfaces.
 */
enum zkwinft_output_device_v1_transform {
	ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_NORMAL = 0,
	ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_90 = 1,
	ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_180 = 2,
	ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_270 = 3,
	ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_FLIPPED = 4,
	ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_FLIPPED_90 = 5,
	ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_FLIPPED_180 = 6,
	ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_FLIPPED_270 = 7,
};
#endif /* ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_ENUM */

#define ZKWINFT_OUTPUT_DEVICE_V1_INFO 0
#define ZKWINFT_OUTPUT_DEVICE_V1_ENABLED 1
#define ZKWINFT_OUTPUT_DEVICE_V1_MODE 2
#define ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM 3
#define ZKWINFT_OUTPUT_DEVICE_V1_GEOMETRY 4
#define ZKWINFT_OUTPUT_DEVICE_V1_DONE 5

/**
 * @ingroup iface_zkwinft_output_device_v1
 */
#define ZKWINFT_OUTPUT_DEVICE_V1_INFO_SINCE_VERSION 1
/**
 * @ingroup iface_zkwinft_output_device_v1
 */
#define ZKWINFT_OUTPUT_DEVICE_V1_ENABLED_SINCE_VERSION 1
/**
 * @ingroup iface_zkwinft_output_device_v1
 */
#define ZKWINFT_OUTPUT_DEVICE_V1_MODE_SINCE_VERSION 1
/**
 * @ingroup iface_zkwinft_output_device_v1
 */
#define ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM_SINCE_VERSION 1
/**
 * @ingroup iface_zkwinft_output_device_v1
 */
#define ZKWINFT_OUTPUT_DEVICE_V1_GEOMETRY_SINCE_VERSION 1
/**
 * @ingroup iface_zkwinft_output_device_v1
 */
#define ZKWINFT_OUTPUT_DEVICE_V1_DONE_SINCE_VERSION 1


/**
 * @ingroup iface_zkwinft_output_device_v1
 * Sends an info event to the client owning the resource.
 * @param resource_ The client's resource
 * @param name unique output identifier
 * @param description human-readable pre-translated device description
 * @param make textual description of the manufacturer
 * @param model textual description of the model
 * @param serial_number serial number of the product
 * @param physical_width width in millimeters of the output
 * @param physical_height height in millimeters of the output
 */
static inline void
zkwinft_output_device_v1_send_info(struct wl_resource *resource_, const char *name, const char *description, const char *make, const char *model, const char *serial_number, int32_t physical_width, int32_t physical_height)
{
	wl_resource_post_event(resource_, ZKWINFT_OUTPUT_DEVICE_V1_INFO, name, description, make, model, serial_number, physical_width, physical_height);
}

/**
 * @ingroup iface_zkwinft_output_device_v1
 * Sends an enabled event to the client owning the resource.
 * @param resource_ The client's resource
 * @param enabled output enabled state
 */
static inline void
zkwinft_output_device_v1_send_enabled(struct wl_resource *resource_, int32_t enabled)
{
	wl_resource_post_event(resource_, ZKWINFT_OUTPUT_DEVICE_V1_ENABLED, enabled);
}

/**
 * @ingroup iface_zkwinft_output_device_v1
 * Sends an mode event to the client owning the resource.
 * @param resource_ The client's resource
 * @param flags bitfield of mode flags
 * @param width width of the mode in hardware units
 * @param height height of the mode in hardware units
 * @param refresh vertical refresh rate in mHz
 * @param mode_id Per output_device unique id to identify a mode
 */
static inline void
zkwinft_output_device_v1_send_mode(struct wl_resource *resource_, uint32_t flags, int32_t width, int32_t height, int32_t refresh, int32_t mode_id)
{
	wl_resource_post_event(resource_, ZKWINFT_OUTPUT_DEVICE_V1_MODE, flags, width, height, refresh, mode_id);
}

/**
 * @ingroup iface_zkwinft_output_device_v1
 * Sends an transform event to the client owning the resource.
 * @param resource_ The client's resource
 */
static inline void
zkwinft_output_device_v1_send_transform(struct wl_resource *resource_, int32_t transform)
{
	wl_resource_post_event(resource_, ZKWINFT_OUTPUT_DEVICE_V1_TRANSFORM, transform);
}

/**
 * @ingroup iface_zkwinft_output_device_v1
 * Sends an geometry event to the client owning the resource.
 * @param resource_ The client's resource
 * @param x x position within the global compositor space
 * @param y y position within the global compositor space
 * @param width logical width of the output in global compositor space
 * @param height logical height of the output in global compositor space
 */
static inline void
zkwinft_output_device_v1_send_geometry(struct wl_resource *resource_, wl_fixed_t x, wl_fixed_t y, wl_fixed_t width, wl_fixed_t height)
{
	wl_resource_post_event(resource_, ZKWINFT_OUTPUT_DEVICE_V1_GEOMETRY, x, y, width, height);
}

/**
 * @ingroup iface_zkwinft_output_device_v1
 * Sends an done event to the client owning the resource.
 * @param resource_ The client's resource
 */
static inline void
zkwinft_output_device_v1_send_done(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, ZKWINFT_OUTPUT_DEVICE_V1_DONE);
}

#ifdef  __cplusplus
}
#endif

#endif
